// Copyright (C) 2005-2007 Stefan Bauer <yoltia-maintain@freenet.de>

// This file is part of Yoltia.
//
// Yoltia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License Version 2
// as published by the Free Software Foundation.
//    
// Yoltia is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Yoltia; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#include "skompvs.hpp"
#include <qsettings.h>
#include <qfileinfo.h>
#include <qlayout.h>
#include <qpushbutton.h>

#include <cstdlib>

const QString KVS = QString("/tmp/kvs-") + QString(getenv("USER"));

SKompVS::SKompVS(QWidget *parent , QImage *dataAddr , QImage *backupAddr) : QDialog(parent , "kompVS" , true) {
  data = dataAddr;
  backup = backupAddr;
  QGridLayout *container = new QGridLayout(this , 4 , 2 , 10 , -1 , "container");
  komprimizer = new QSpinBox(0 , 100 , 1 , this , "komprimizer");
  container->addWidget(komprimizer , 0 , 1);
  container->addWidget(new QLabel(komprimizer ,  tr("&Compression level:") , this , "descrComp") , 0 , 0);
  fileSize = new QLabel(this , "fileSize");
  container->addWidget(fileSize , 1 , 1);
  container->addWidget(new QLabel(tr("File size:") , this , "descrFileSize") , 1 , 0);
  format = new QListBox(this , "format");
  format->insertStringList(QStringList::split("-" , "JPEG-PNG"));
  format->setCurrentItem(0);
  format->setSizePolicy(QSizePolicy::Minimum , QSizePolicy::Minimum);
  container->addWidget(format , 2 , 1);
  container->addWidget(new QLabel(format , tr("F&ormat:") , this , "descrFormat") , 2 , 0); 
  QPushButton *ok = new QPushButton(tr("&Finished") , this , "ok");
  ok->setDefault(true);
  container->addMultiCellWidget(ok , 3 , 3 , 0 , 1);
  connect(komprimizer , SIGNAL(valueChanged(int)) , this , SLOT(createPreview(int)));
  connect (format , SIGNAL(highlighted(int)) , this , SLOT(changeFormat(int)));
  connect(ok , SIGNAL(clicked()) , this , SLOT(quit()));
  komprimizer->setValue(100 - QSettings().readNumEntry("/Yoltia/Quality/JPEG" , 0));
  setCaption(tr("Compression preview"));
}

void SKompVS::createPreview(int komp) {
  if (format->isSelected(0)) {	//Format == JPEG?
      backup->save(KVS , "JPEG" , 100 - komp);
      fileSize->setText(QString::number(QFileInfo(KVS).size() / 1024.0) + " kB");
      data->load(KVS);
      emit modified();
    } else {	//sonst (Format == PNG)
      backup->save(KVS , "PNG" , 100 - 10 * komp);
      fileSize->setText(QString::number(QFileInfo(KVS).size() / 1024.0) + " kB");	//da PNG verlustfrei, w�re ein erneutes Laden nur Zeitverschwendung
  }
}

void SKompVS::changeFormat(int newFormat) {
  if (newFormat)	//Format == PNG?
      komprimizer->setRange(1 , 10);
    else	//sonst (Format == JPEG)
      komprimizer->setRange(0 , 100);
  createPreview(komprimizer->value());
}

void SKompVS::quit() {
  if (format->isSelected(0))	//Format == JPEG?
      QSettings().writeEntry("/Yoltia/Quality/JPEG" , 100 - komprimizer->value());
    else	//sonst (Format == PNG)
      QSettings().writeEntry("/Yoltia/Quality/PNG" , 100 - 10 * komprimizer->value());
  QFile::remove(KVS);
  accept();
}
