// Copyright (C) 2005-2007 Stefan Bauer <yoltia-maintain@freenet.de>

// This file is part of Yoltia.
//
// Yoltia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License Version 2
// as published by the Free Software Foundation.
//    
// Yoltia is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Yoltia; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#include "sset.hpp"
#include <qlayout.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qsettings.h>
#include <qfiledialog.h>
#include <qcolordialog.h>

SSet::SSet(QWidget *parent) : QDialog(parent , "set" , true) {
  QVBoxLayout *container = new QVBoxLayout(this , 10 , -1 , "container");
  toolBox = new QToolBox(this , "toolBox");
  container->addWidget(toolBox);
  QWidget *misc = new QWidget(this , "misc");	//Misc Settings
  QGridLayout *miscLayout = new QGridLayout(misc , 4 , 2 , 10 , -1 , "miscLayout");
  QLabel *descrUndoN = new QLabel(tr("Count of &Undos:") , misc , "descrUndoN");
  QLabel *descrIntroN = new QLabel(tr("Count of files for &Intro:") , misc , "descrIntroN");
  QLabel *descrColTol = new QLabel(tr("&Color tolerance for removing margin:") , misc , "descColTol");
  QLabel *descrMarkspace = new QLabel(tr("Free &space around image:") , misc , "descMarkspace");
  undoN = new QSpinBox(1 , 127 , 1 , misc , "undoN");
  introN = new QSpinBox(1 , INT_MAX , 1 , misc , "introN");	// INT_MAX = 2147483647
  colTol = new QSpinBox(0 , 254 , 1 , misc , "colTol");
  markspace = new QSpinBox(0 , 20 , 1 , misc , "markspace");
  markspace->setSuffix(" Pixel");
  descrUndoN->setBuddy(undoN);
  descrIntroN->setBuddy(introN);
  descrColTol->setBuddy(colTol);
  descrMarkspace->setBuddy(markspace);
  miscLayout->addWidget(descrUndoN , 0 , 0);
  miscLayout->addWidget(descrIntroN , 1 , 0);
  miscLayout->addWidget(descrColTol , 2 , 0);
  miscLayout->addWidget(descrMarkspace , 3 , 0);
  miscLayout->addWidget(undoN , 0 , 1);
  miscLayout->addWidget(introN , 1 , 1);
  miscLayout->addWidget(colTol , 2 , 1);
  miscLayout->addWidget(markspace , 3 , 1);
  toolBox->addItem(misc , tr("&Miscellaneous"));
  QWidget *ss = new QWidget(this , "ss");	//Slideshow Settings
  QGridLayout *ssLayout = new QGridLayout(ss , 5 , 2 , 10 , -1 , "ssLayout");
  ssTimer = new QCheckBox(tr("&Use timer") , ss , "ssTimer");
  ssTime = new QSpinBox(1000 , 86400000 , 1 , ss , "ssTime");	//1 second to 1 day
  ssTime->setSuffix(" ms");
  ssRandom = new QCheckBox(tr("&Random order") , ss , "ssRandom");
  ssRepeat = new QCheckBox(tr("Re&peat when finished") , ss , "ssRepeat");
  ssDisplayFilename = new QCheckBox(tr("Display file&name") , ss , "ssDisplayFilename");
  ssDisplayListpos = new QCheckBox(tr("Display &position in filelist") , ss , "ssDisplayListpos");
  ssTimer->setChecked(true);	//Need to set checked first to disable ssTime when loading settings
  connect(ssTimer , SIGNAL(toggled(bool)) , ssTime , SLOT(setEnabled(bool)));
  connect(ssRandom , SIGNAL(toggled(bool)) , ssRepeat , SLOT(setDisabled(bool)));
  ssLayout->addWidget(ssTimer , 0 , 0);
  ssLayout->addWidget(ssTime , 0 , 1);
  ssLayout->addMultiCellWidget(ssRandom , 1 , 1 , 0 , 1);
  ssLayout->addMultiCellWidget(ssRepeat , 2 , 2 , 0 , 1);
  ssLayout->addMultiCellWidget(ssDisplayFilename , 3 , 3 , 0 , 1);
  ssLayout->addMultiCellWidget(ssDisplayListpos , 4 , 4 , 0 , 1);
  toolBox->addItem(ss , tr("&Slideshow"));
  QWidget *history = new QWidget(this , "history");	//Histroy Settings
  QGridLayout *historyLayout = new QGridLayout(history , 3 , 2 , 10 , -1 , "historyLayout");
  show = new QCheckBox(tr("&Show list in Pictures menu") , history , "show");
  QLabel *descrHistoryN = new QLabel(tr("Count of &entries:") , history , "descrHistoryN");
  historyN = new QSpinBox(1 , 100 , 1 , history , "historyN");
  descrHistoryN->setBuddy(historyN);
  QPushButton *clear = new QPushButton(tr("Remove &all entries") , history , "clear");
  connect(clear , SIGNAL(clicked()) , this , SLOT(clearHistory()));
  historyLayout->addMultiCellWidget(show , 0 , 0 , 0 , 1);
  historyLayout->addWidget(descrHistoryN , 1 , 0);
  historyLayout->addWidget(historyN , 1 , 1);
  historyLayout->addMultiCellWidget(clear , 2 , 2 , 0 , 1);
  toolBox->addItem(history , tr("&File history"));
  QWidget *komp = new QWidget(this , "komp");	//Compression Settings
  QGridLayout *kompLayout = new QGridLayout(komp , 2 , 2 , 10 , -1 , "kompLayout");
  QLabel *descrJPEG = new QLabel("&JPEG:" , komp , "descrJPEG");
  QLabel *descrPNG = new QLabel("&PNG:" , komp , "descrPNG");
  kompJPEG = new QSpinBox(0 , 100 , 1 , komp , "kompJPEG");
  kompPNG = new QSpinBox(1 , 10 , 1 , komp , "kompPNG");
  descrJPEG->setBuddy(kompJPEG);
  descrPNG->setBuddy(kompPNG);
  kompLayout->addWidget(descrJPEG , 0 , 0);
  kompLayout->addWidget(descrPNG , 1 , 0);
  kompLayout->addWidget(kompJPEG , 0 , 1);
  kompLayout->addWidget(kompPNG , 1 , 1);
  toolBox->addItem(komp , tr("&Compression"));
  QWidget *edgeCut = new QWidget(this , "edgeCut");	//Cutting edges settings
  QGridLayout *edgeCutLayout = new QGridLayout(edgeCut , 3 , 2 , 10 , -1 , "edgeCutLayout");
  QLabel *descrEdgeWO = new QLabel(tr("&Arrows only:") , edgeCut , "descrEdgeWO");
  QLabel *descrEdgeCtrl = new QLabel(tr("&Ctrl + arrows:") , edgeCut , "descrEdgeCtrl");
  QLabel *descrEdgeShift = new QLabel(tr("&Shift + arrows:") , edgeCut , "descrEdgeShift");
  edgeWO = new QSpinBox(1 , 4000 , 1 , edgeCut , "edgeWO");
  edgeCtrl = new QSpinBox(1 , 4000 , 1 , edgeCut , "edgeCtrl");
  edgeShift = new QSpinBox(1 , 4000 , 1 , edgeCut , "edgeShift");
  edgeWO->setSuffix(" Pixel");
  edgeCtrl->setSuffix(" Pixel");
  edgeShift->setSuffix(" Pixel");
  descrEdgeWO->setBuddy(edgeWO);
  descrEdgeCtrl->setBuddy(edgeCtrl);
  descrEdgeShift->setBuddy(edgeShift);
  edgeCutLayout->addWidget(descrEdgeWO , 0 , 0);
  edgeCutLayout->addWidget(descrEdgeCtrl , 1 , 0);
  edgeCutLayout->addWidget(descrEdgeShift , 2 , 0);
  edgeCutLayout->addWidget(edgeWO , 0 , 1);
  edgeCutLayout->addWidget(edgeCtrl , 1 , 1);
  edgeCutLayout->addWidget(edgeShift , 2 , 1);
  toolBox->addItem(edgeCut , tr("&Edge cut"));
  QWidget *watch = new QWidget(this , "watch");	//Watch Settings
  QGridLayout *watchLayout = new QGridLayout(watch , 8 , 2 , 10 , -1 , "watchLayout");
  watchWidthMin = new QCheckBox(tr("M&inimum width") , watch , "watchWidthMin");
  watchWidthMax = new QCheckBox(tr("M&aximum width") , watch , "watchWidthMax");
  watchHeightMin = new QCheckBox(tr("M&inimum height") , watch , "watchHeightMin");
  watchHeightMax = new QCheckBox(tr("M&aximum height") , watch , "watchHeightMax");
  watchCountMin = new QCheckBox(tr("M&inimum pixel count") , watch , "watchCountMin");
  watchCountMax = new QCheckBox(tr("M&aximum pixel count") , watch , "watchCountMax");
  watchBeep = new QCheckBox(tr("&Beep at violation") , watch , "watchBeep");
  watchColor = new QCheckBox(tr("Change background c&olor at violation") , watch , "watchColor");
  QPushButton *colorSelector = new QPushButton(tr("Se&lect color...") , watch , "colorSelector"); 
  watchWidthMin->setChecked(true);	//Need to set checked first to disable the Spinboxes when loading settings
  watchWidthMax->setChecked(true);
  watchHeightMin->setChecked(true);
  watchHeightMax->setChecked(true);
  watchCountMin->setChecked(true);
  watchCountMax->setChecked(true);
  watchColor->setChecked(true);
  widthMinValue = new QSpinBox(2 , 8200 , 1 , watch , "widthMinValue");
  widthMaxValue = new QSpinBox(1 , 8199 , 1 , watch , "widthMaxValue");
  heightMinValue = new QSpinBox(2 , 8200 , 1 , watch , "heightMinValue");
  heightMaxValue = new QSpinBox(1 , 8199 , 1 , watch , "heightMaxValue");
  countMinValue = new QSpinBox(2 , 67223601 , 1 , watch , "countMinValue");
  countMaxValue = new QSpinBox(1 , 67223601 , 1 , watch , "countMaxValue");
  widthMinValue->setSuffix(" Pixel");
  widthMaxValue->setSuffix(" Pixel");
  heightMinValue->setSuffix(" Pixel");
  heightMaxValue->setSuffix(" Pixel");
  countMinValue->setSuffix(" Pixel");
  countMaxValue->setSuffix(" Pixel");
  connect(watchWidthMin , SIGNAL(toggled(bool)) , widthMinValue , SLOT(setEnabled(bool)));
  connect(watchWidthMax , SIGNAL(toggled(bool)) , widthMaxValue , SLOT(setEnabled(bool)));
  connect(watchHeightMin , SIGNAL(toggled(bool)) , heightMinValue , SLOT(setEnabled(bool)));
  connect(watchHeightMax , SIGNAL(toggled(bool)) , heightMaxValue , SLOT(setEnabled(bool)));
  connect(watchCountMin , SIGNAL(toggled(bool)) , countMinValue , SLOT(setEnabled(bool)));
  connect(watchCountMax , SIGNAL(toggled(bool)) , countMaxValue , SLOT(setEnabled(bool)));
  connect(watchColor , SIGNAL(toggled(bool)) , colorSelector , SLOT(setEnabled(bool)));
  connect(colorSelector , SIGNAL(clicked()) , this , SLOT(selectColor()));
  watchLayout->addWidget(watchWidthMin , 0 , 0);
  watchLayout->addWidget(watchWidthMax , 1 , 0);
  watchLayout->addWidget(watchHeightMin , 2 , 0);
  watchLayout->addWidget(watchHeightMax , 3 , 0);
  watchLayout->addWidget(watchCountMin , 4 , 0);
  watchLayout->addWidget(watchCountMax , 5 , 0);
  watchLayout->addWidget(widthMinValue , 0 , 1);
  watchLayout->addWidget(widthMaxValue , 1 , 1);
  watchLayout->addWidget(heightMinValue , 2 , 1);
  watchLayout->addWidget(heightMaxValue , 3 , 1);
  watchLayout->addWidget(countMinValue , 4 , 1);
  watchLayout->addWidget(countMaxValue , 5 , 1);
  watchLayout->addMultiCellWidget(watchBeep , 6 , 6 , 0 , 1);
  watchLayout->addWidget(watchColor , 7 , 0);
  watchLayout->addWidget(colorSelector , 7 , 1);
  toolBox->addItem(watch , tr("&Watch"));
  QWidget *trans = new QWidget(this , "trans");	//Internationalization Settings
  QGridLayout *transLayout = new QGridLayout(trans , 6 , 2 , 10 , -1 , "transLayout");
  QLabel *descrQtTrans = new QLabel(tr("Qt translation file:") , trans , "descrQtTrans");
  QLabel *descrYoltiaTrans = new QLabel(tr("Yoltia translation file:") , trans , "descrYoltiaTrans");
  qtTrans = new QLabel(trans , "qtTrans");
  yoltiaTrans = new QLabel(trans , "yoltiaTrans");
  QPushButton *qtUnset = new QPushButton(tr("&None") , trans , "qtUnset");
  QPushButton *yoltiaUnset = new QPushButton(tr("&None") , trans , "yoltiaUnset");
  QPushButton *qtSearch = new QPushButton(tr("Se&lect...") , trans , "qtSearch");
  QPushButton *yoltiaSearch = new QPushButton(tr("Se&lect...") , trans , "yoltiaSearch");
  connect(qtUnset , SIGNAL(clicked()) , this , SLOT(unsetQtTrans()));
  connect(yoltiaUnset , SIGNAL(clicked()) , this , SLOT(unsetYoltiaTrans()));
  connect(qtSearch , SIGNAL(clicked()) , this , SLOT(searchQtTrans()));
  connect(yoltiaSearch , SIGNAL(clicked()) , this , SLOT(searchYoltiaTrans()));
  transLayout->addMultiCellWidget(descrQtTrans , 0 , 0 , 0 , 1);
  transLayout->addMultiCellWidget(qtTrans , 1 , 1 , 0 , 1);
  transLayout->addWidget(qtUnset , 2 , 0);
  transLayout->addWidget(qtSearch , 2 , 1);
  transLayout->addMultiCellWidget(descrYoltiaTrans , 3 , 3 , 0 , 1);
  transLayout->addMultiCellWidget(yoltiaTrans , 4 , 4 , 0 , 1);
  transLayout->addWidget(yoltiaUnset , 5 , 0);
  transLayout->addWidget(yoltiaSearch , 5 , 1);
  toolBox->addItem(trans , tr("&Internationalization"));
  QSettings set;	//Read settings and set to widgets
  toolBox->setCurrentIndex(set.readNumEntry("/Yoltia/setTab" , 0));
  undoN->setValue(set.readNumEntry("/Yoltia/undoN" , 10));
  introN->setValue(set.readNumEntry("/Yoltia/introN" , 2));
  colTol->setValue(set.readNumEntry("/Yoltia/colTol" , 20));
  markspace->setValue(set.readNumEntry("/Yoltia/markspace" , 15));
  ssTimer->setChecked(set.readBoolEntry("/Yoltia/ss/timer" , true));
  ssTime->setValue(set.readNumEntry("/Yoltia/ss/time" , 10000));
  ssRandom->setChecked(set.readBoolEntry("/Yoltia/ss/random" , false));
  ssRepeat->setChecked(set.readBoolEntry("/Yoltia/ss/repeat" , false));
  ssDisplayFilename->setChecked(set.readBoolEntry("/Yoltia/ss/displayFilename" , false));
  ssDisplayListpos->setChecked(set.readBoolEntry("/Yoltia/ss/displayListpos" , false));
  show->setChecked(set.readBoolEntry("/Yoltia/history/show" , true));
  historyN->setValue(set.readNumEntry("/Yoltia/history/count" , 10));
  kompJPEG->setValue(100 - set.readNumEntry("/Yoltia/Quality/JPEG" , 100));
  kompPNG->setValue(10 - qRound(set.readNumEntry("/Yoltia/Quality/PNG" , 0) / 10.0));
  edgeWO->setValue(set.readNumEntry("/Yoltia/Edge/WO" , 5));
  edgeCtrl->setValue(set.readNumEntry("/Yoltia/Edge/Ctrl" , 1));
  edgeShift->setValue(set.readNumEntry("/Yoltia/Edge/Shift" , 20));
  watchWidthMin->setChecked(set.readBoolEntry("/Yoltia/Watch/WatchWidthMin" , false));
  watchWidthMax->setChecked(set.readBoolEntry("/Yoltia/Watch/WatchWidthMax" , false));
  watchHeightMin->setChecked(set.readBoolEntry("/Yoltia/Watch/WatchHeightMin" , false));
  watchHeightMax->setChecked(set.readBoolEntry("/Yoltia/Watch/WatchHeightMax" , false));
  watchCountMin->setChecked(set.readBoolEntry("/Yoltia/Watch/WatchCountMin" , false));
  watchCountMax->setChecked(set.readBoolEntry("/Yoltia/Watch/WatchCountMax" , false));
  watchBeep->setChecked(set.readBoolEntry("/Yoltia/Watch/beep" , false));
  watchColor->setChecked(set.readBoolEntry("/Yoltia/Watch/changeColor" , false));
  backgroundColor = QColor(set.readEntry("/Yoltia/Watch/color" , "#ff0000"));
  widthMinValue->setValue(set.readNumEntry("/Yoltia/Watch/widthMinValue" , 100));
  widthMaxValue->setValue(set.readNumEntry("/Yoltia/Watch/widthMaxValue" , 100));
  heightMinValue->setValue(set.readNumEntry("/Yoltia/Watch/heightMinValue" , 100));
  heightMaxValue->setValue(set.readNumEntry("/Yoltia/Watch/heightMaxValue" , 100));
  countMinValue->setValue(set.readNumEntry("/Yoltia/Watch/countMinValue" , 10000));
  countMaxValue->setValue(set.readNumEntry("/Yoltia/Watch/countMaxValue" , 10000));
  qtTrans->setText(set.readEntry("/Yoltia/Translation/qt"));
  yoltiaTrans->setText(set.readEntry("/Yoltia/Translation/yoltia"));
  QPushButton *ok = new QPushButton(tr("&Finished") , this , "ok");
  ok->setDefault(true);
  container->addWidget(ok);
  connect(ok , SIGNAL(clicked()) , this , SLOT(quit()));
  setCaption(tr("Settings"));
}

void SSet::clearHistory() {
  QSettings set;
  set.writeEntry("/Yoltia/history/list" , (QStringList) 0);
  set.writeEntry("/Yoltia/history/dir" , (QString) 0);
}

void SSet::quit() {
  QSettings set;
  set.writeEntry("/Yoltia/setTab" , toolBox->currentIndex());
  set.writeEntry("/Yoltia/undoN" , undoN->value());
  set.writeEntry("/Yoltia/introN" , introN->value());
  set.writeEntry("/Yoltia/colTol" , colTol->value());
  set.writeEntry("/Yoltia/markspace" , markspace->value());
  set.writeEntry("/Yoltia/ss/timer" , ssTimer->isChecked());
  set.writeEntry("/Yoltia/ss/time" , ssTime->value());
  set.writeEntry("/Yoltia/ss/random" , ssRandom->isChecked());
  set.writeEntry("/Yoltia/ss/repeat" , ssRepeat->isChecked());
  set.writeEntry("/Yoltia/ss/displayFilename" , ssDisplayFilename->isChecked());
  set.writeEntry("/Yoltia/ss/displayListpos" , ssDisplayListpos->isChecked());
  set.writeEntry("/Yoltia/history/show" , show->isChecked());
  set.writeEntry("/Yoltia/history/count" , historyN->value());
  set.writeEntry("/Yoltia/Quality/JPEG" , 100 - kompJPEG->value());
  set.writeEntry("/Yoltia/Quality/PNG" , 100 - 10 * kompPNG->value());
  set.writeEntry("/Yoltia/Edge/WO" , edgeWO->value());
  set.writeEntry("/Yoltia/Edge/Ctrl" , edgeCtrl->value());
  set.writeEntry("/Yoltia/Edge/Shift" , edgeShift->value());
  set.writeEntry("/Yoltia/Watch/WatchWidthMin" , watchWidthMin->isChecked());
  set.writeEntry("/Yoltia/Watch/WatchWidthMax" , watchWidthMax->isChecked());
  set.writeEntry("/Yoltia/Watch/WatchHeightMin" , watchHeightMin->isChecked());
  set.writeEntry("/Yoltia/Watch/WatchHeightMax" , watchHeightMax->isChecked());
  set.writeEntry("/Yoltia/Watch/WatchCountMin" , watchCountMin->isChecked());
  set.writeEntry("/Yoltia/Watch/WatchCountMax" , watchCountMax->isChecked());
  set.writeEntry("/Yoltia/Watch/widthMinValue" , widthMinValue->value());
  set.writeEntry("/Yoltia/Watch/widthMaxValue" , widthMaxValue->value());
  set.writeEntry("/Yoltia/Watch/heightMinValue" , heightMinValue->value());
  set.writeEntry("/Yoltia/Watch/heightMaxValue" , heightMaxValue->value());
  set.writeEntry("/Yoltia/Watch/countMinValue" , countMinValue->value());
  set.writeEntry("/Yoltia/Watch/countMaxValue" , countMaxValue->value());
  set.writeEntry("/Yoltia/Watch/beep" , watchBeep->isChecked());
  set.writeEntry("/Yoltia/Watch/changeColor" , watchColor->isChecked());
  set.writeEntry("/Yoltia/Watch/color" , backgroundColor.name());
  set.writeEntry("/Yoltia/Translation/qt" , qtTrans->text());
  set.writeEntry("/Yoltia/Translation/yoltia" , yoltiaTrans->text());
  accept();
}

void SSet::unsetQtTrans() {
  qtTrans->setText(0);
}

void SSet::unsetYoltiaTrans() {
  yoltiaTrans->setText(0);
}

void SSet::searchQtTrans() {
  QString newTrans(QFileDialog::getOpenFileName("/usr/lib/qt3/translations/" , "Qt Message Files (*.qm)" , this , "qtTransDlg" , tr("Choose file")));
  if (newTrans)
    qtTrans->setText(newTrans);
}

void SSet::searchYoltiaTrans() {
  QString newTrans(QFileDialog::getOpenFileName("/usr/share/yoltia/" , "Qt Message Files (*.qm)" , this , "yoltiaTransDlg" , tr("Choose file")));
  if (newTrans)
    yoltiaTrans->setText(newTrans);
}

void SSet::selectColor() {
  backgroundColor = QColorDialog::getColor(backgroundColor , this , "colorSelectionDialog");
}
