// Copyright (C) 2005-2007 Stefan Bauer <yoltia-maintain@freenet.de>

// This file is part of Yoltia.
//
// Yoltia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License Version 2
// as published by the Free Software Foundation.
//    
// Yoltia is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Yoltia; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#include "sworkspace.hpp"
#include <qdragobject.h>

SWorkspace::SWorkspace(QWidget *parent) : QWorkspace(parent , "worksp") {
  setAcceptDrops(true);
}

void SWorkspace::hTile() {
  int newHeight = height() / windowList().count() , y = 0;
  QWidget *w;
  for (uchar i = 0 ; i < windowList().count() ; i++) {
    w = windowList().at(i);
    if (w->isMaximized())
      w->showNormal();
    w->parentWidget()->setGeometry(0 , y , width() , newHeight);
    y += newHeight;
  }
}

void SWorkspace::minimize() {
  for (uchar i = 0 ; i < windowList().count() ; i++)
    windowList().at(i)->showMinimized();
}

void SWorkspace::dragEnterEvent(QDragEnterEvent *event) {
  event->accept(QTextDrag::canDecode(event));
}

void SWorkspace::dropEvent(QDropEvent *event) {
  QString text;
  if (QTextDrag::decode(event, text)) {
    QStringList list = QStringList::split("\n" , text);
    for (uint i = 0 ; i < list.count() ; i++)
      list[i].remove("file:");	//Konqueror drops are in format <file:><path to file>
    if (list.count() == 1)	//Decide weather to use custom-list- or directory-mode for new SPicView
        emit fileDropped(list[0]);
      else
        emit listDropped(list);
  }
}

void SWorkspace::closeAll() {
  for (int zv = windowList().count() - 1 ; zv >= 0 ; zv--) {
    windowList().at(zv)->showNormal();
    windowList().at(zv)->close();
  }
}
